import { __rest } from 'tslib';
import { HtmlAttributePlugin, application } from '@visactor/vtable/es/vrender';
import { isNil, uniqArray, isEqual, isObject, isArray, calculateAnchorOfBounds, isFunction, isString, styleStringToObject } from '@visactor/vutils';
import { render } from 'vue';

class VTableVueAttributePlugin extends HtmlAttributePlugin {
    constructor(currentContext) {
        super();
        this.name = 'VTableVueAttributePlugin';
        this.renderQueue = new Set();
        this.isRendering = false;
        this.currentContext = currentContext;
    }
    renderGraphicHTML(graphic) {
        if (!this.checkNeedRender(graphic)) {
            return;
        }
        this.renderQueue.add(graphic);
        this.scheduleRender();
    }
    scheduleRender() {
        if (this.isRendering) {
            return;
        }
        this.isRendering = true;
        requestAnimationFrame(() => {
            this.renderQueue.forEach(graphic => {
                try {
                    this.doRenderGraphic(graphic);
                }
                catch (error) {
                    const { id } = this.getGraphicOptions(graphic) || {};
                    this.removeElement(id, true);
                }
            });
            this.renderQueue.clear();
            this.isRendering = false;
        });
    }
    doRenderGraphic(graphic) {
        var _a;
        const { id, options } = this.getGraphicOptions(graphic);
        if (!id) {
            return;
        }
        const stage = graphic.stage;
        const { element, container: expectedContainer } = options;
        const actualContainer = expectedContainer ? checkFrozenContainer(graphic) : expectedContainer;
        let targetMap = (_a = this.htmlMap) === null || _a === void 0 ? void 0 : _a[id];
        if (targetMap && actualContainer && actualContainer !== targetMap.container) {
            this.removeElement(id);
            targetMap = null;
        }
        this.checkToPassAppContext(element, graphic);
        if (!targetMap || !this.checkDom(targetMap.wrapContainer)) {
            const { wrapContainer, nativeContainer } = this.getWrapContainer(stage, actualContainer, { id, options });
            if (wrapContainer) {
                const dataRenderId = `${this.renderId}`;
                wrapContainer.id = id;
                wrapContainer.setAttribute('data-vue-renderId', dataRenderId);
                render(element, wrapContainer);
                targetMap = {
                    wrapContainer,
                    nativeContainer,
                    container: actualContainer,
                    renderId: this.renderId,
                    graphic,
                    isInViewport: true,
                    lastPosition: null,
                    lastStyle: {}
                };
                this.htmlMap[id] = targetMap;
            }
        }
        else {
            render(element, targetMap.wrapContainer);
        }
        if (targetMap) {
            this.updateStyleOfWrapContainer(graphic, stage, targetMap.wrapContainer, targetMap.nativeContainer);
            targetMap.renderId = this.renderId;
        }
    }
    getGraphicOptions(graphic) {
        var _a;
        const { vue } = (graphic === null || graphic === void 0 ? void 0 : graphic.attribute) || {};
        if (!vue) {
            return null;
        }
        const id = isNil(vue.id) ? (_a = graphic.id) !== null && _a !== void 0 ? _a : graphic._uid : vue.id;
        return { id: `vue_${id}`, options: vue };
    }
    checkToPassAppContext(vnode, graphic) {
        var _a, _b, _c, _d;
        try {
            const { stage } = getTargetGroup(graphic);
            const { table } = stage || {};
            const userAppContext = (_d = (_c = (_b = (_a = table === null || table === void 0 ? void 0 : table.options) === null || _a === void 0 ? void 0 : _a.customConfig) === null || _b === void 0 ? void 0 : _b.getVueUserAppContext) === null || _c === void 0 ? void 0 : _c.call(_b)) !== null && _d !== void 0 ? _d : this.currentContext;
            if (!!(userAppContext === null || userAppContext === void 0 ? void 0 : userAppContext.components) && !!(userAppContext === null || userAppContext === void 0 ? void 0 : userAppContext.directives)) {
                vnode.appContext = userAppContext;
            }
        }
        catch (error) { }
    }
    checkNeedRender(graphic) {
        const { id, options } = this.getGraphicOptions(graphic) || {};
        if (!id) {
            return false;
        }
        const stage = graphic.stage;
        if (!stage) {
            return false;
        }
        const { element } = options;
        if (!element) {
            return false;
        }
        const isInViewport = this.checkInViewport(graphic);
        return isInViewport;
    }
    checkInViewport(graphic) {
        const { stage, globalAABBBounds: cBounds } = graphic;
        if (!stage) {
            return false;
        }
        const BUFFER = 100;
        const { AABBBounds: vBounds } = stage;
        const eBounds = {
            x1: vBounds.x1 - BUFFER,
            x2: vBounds.x2 + BUFFER,
            y1: vBounds.y1 - BUFFER,
            y2: vBounds.y2 + BUFFER
        };
        const isIntersecting = cBounds.x1 < eBounds.x2 && cBounds.x2 > eBounds.x1 && cBounds.y1 < eBounds.y2 && cBounds.y2 > eBounds.y1;
        return isIntersecting;
    }
    checkDom(dom) {
        if (!dom) {
            return false;
        }
        return document.contains(dom);
    }
    removeAllDom(g) {
        if (this.htmlMap) {
            Object.keys(this.htmlMap).forEach(key => {
                this.removeElement(key, true);
            });
            this.htmlMap = null;
        }
    }
    removeElement(id, clear) {
        var _a;
        const record = (_a = this.htmlMap) === null || _a === void 0 ? void 0 : _a[id];
        if (!record) {
            return;
        }
        const { wrapContainer } = record;
        if (!wrapContainer) {
            return;
        }
        render(null, wrapContainer);
        if (!clear) {
            wrapContainer.remove();
            record.isInViewport = false;
        }
        else {
            this.checkDom(wrapContainer) && super.removeElement(id);
            delete this.htmlMap[id];
        }
        this.removeWrapContainerEventListener(wrapContainer);
    }
    getWrapContainer(stage, userContainer, domParams) {
        var _a;
        let nativeContainer;
        if (userContainer) {
            nativeContainer =
                typeof userContainer === 'string' ? application.global.getElementById(userContainer) : userContainer;
        }
        else {
            nativeContainer = stage.window.getContainer();
        }
        const { id } = domParams || {};
        const record = (_a = this.htmlMap) === null || _a === void 0 ? void 0 : _a[id];
        if (record && !record.isInViewport) {
            const { wrapContainer } = record;
            if (!this.checkDom(wrapContainer)) {
                nativeContainer.appendChild(wrapContainer);
            }
            return {
                wrapContainer,
                nativeContainer
            };
        }
        return {
            wrapContainer: application.global.createDom({ tagName: 'div', parent: nativeContainer }),
            nativeContainer
        };
    }
    updateStyleOfWrapContainer(graphic, stage, wrapContainer, nativeContainer) {
        const { attribute, type } = graphic;
        const _a = attribute || {}, { vue: options, width, height, visible, display } = _a, rest = __rest(_a, ["vue", "width", "height", "visible", "display"]);
        const { x: left, y: top } = this.calculatePosition(graphic, options.anchorType);
        const { left: offsetX, top: offsetTop } = this.calculateOffset(stage, nativeContainer, left, top);
        const { id } = this.getGraphicOptions(graphic) || {};
        const record = id ? this.htmlMap[id] : null;
        if (!record) {
            return;
        }
        const positionChanged = !record.lastPosition || record.lastPosition.x !== offsetX || record.lastPosition.y !== offsetTop;
        if (!positionChanged) {
            return;
        }
        const { pointerEvents, penetrateEventList = ['wheel'] } = options;
        const calculateStyle = this.parseDefaultStyleFromGraphic(graphic);
        const style = this.convertCellStyle(graphic);
        Object.assign(calculateStyle, Object.assign(Object.assign(Object.assign({ overflow: 'hidden' }, (style || {})), (rest || {})), { transform: `translate(${offsetX}px, ${offsetTop}px)`, boxSizing: 'border-box', display: visible !== false ? display || 'block' : 'none', pointerEvents: pointerEvents === true ? 'all' : pointerEvents || 'none', position: 'absolute' }));
        if (calculateStyle.pointerEvents !== 'none') {
            this.removeWrapContainerEventListener(wrapContainer);
            uniqArray(penetrateEventList).forEach((event) => {
                if (event === 'wheel') {
                    wrapContainer.addEventListener('wheel', this.onWheel);
                    wrapContainer.addEventListener('wheel', e => e.preventDefault(), true);
                }
            });
        }
        if (type === 'text' && options.anchorType === 'position') {
            Object.assign(calculateStyle, this.getTransformOfText(graphic));
        }
        this.applyUserStyles(options, calculateStyle, { offsetX, offsetTop, graphic, wrapContainer });
        const styleChanged = !isEqual(record.lastStyle, calculateStyle);
        if (styleChanged) {
            application.global.updateDom(wrapContainer, {
                width,
                height,
                style: calculateStyle
            });
            record.lastStyle = calculateStyle;
        }
    }
    convertCellStyle(graphic) {
        var _a;
        const { col, row, stage } = getTargetGroup(graphic);
        const style = (_a = stage === null || stage === void 0 ? void 0 : stage.table) === null || _a === void 0 ? void 0 : _a.getCellStyle(col, row);
        if (!isObject(style)) {
            return;
        }
        const _b = style, { lineHeight, padding } = _b, rest = __rest(_b, ["lineHeight", "padding"]);
        return Object.assign(Object.assign({}, rest), { padding: isArray(padding) ? padding.map(value => `${value}px`).join(' ') : padding });
    }
    calculatePosition(graphic, anchorType) {
        const bounds = graphic.globalAABBBounds;
        if (anchorType === 'position' || bounds.empty()) {
            const matrix = graphic.globalTransMatrix;
            return { x: matrix.e, y: matrix.f };
        }
        return calculateAnchorOfBounds(bounds, anchorType || 'top-left');
    }
    calculateOffset(stage, nativeContainer, x, y) {
        const containerTL = application.global.getElementTopLeft(nativeContainer, false);
        const windowTL = stage.window.getTopLeft(false);
        return {
            left: x + windowTL.left - containerTL.left,
            top: y + windowTL.top - containerTL.top
        };
    }
    applyUserStyles(options, baseStyle, context) {
        if (isFunction(options.style)) {
            const userStyle = options.style({
                top: context.offsetTop,
                left: context.offsetX,
                width: context.graphic.globalAABBBounds.width(),
                height: context.graphic.globalAABBBounds.height()
            }, context.graphic, context.wrapContainer);
            Object.assign(baseStyle, userStyle);
        }
        else if (isObject(options.style)) {
            Object.assign(baseStyle, options.style);
        }
        else if (isString(options.style)) {
            Object.assign(baseStyle, styleStringToObject(options.style));
        }
    }
}
function checkFrozenContainer(graphic) {
    var _a;
    const { col, row, stage } = getTargetGroup(graphic);
    let container = (_a = graphic.attribute.vue) === null || _a === void 0 ? void 0 : _a.container;
    const { table } = stage;
    if (container === table.bodyDomContainer) {
        if (col < table.frozenColCount && row >= table.rowCount - table.bottomFrozenRowCount) {
            container = table.bottomFrozenBodyDomContainer;
        }
        else if (col >= table.colCount - table.rightFrozenColCount &&
            row >= table.rowCount - table.bottomFrozenRowCount) {
            container = table.rightFrozenBottomDomContainer;
        }
        else if (row >= table.rowCount - table.bottomFrozenRowCount) {
            container = table.bottomFrozenBodyDomContainer;
        }
        else if (col < table.frozenColCount) {
            container = table.frozenBodyDomContainer;
        }
        else if (col >= table.colCount - table.rightFrozenColCount) {
            container = table.rightFrozenBodyDomContainer;
        }
    }
    else if (container === table.headerDomContainer) {
        if (col < table.frozenColCount) {
            container = table.frozenHeaderDomContainer;
        }
        else if (col >= table.colCount - table.rightFrozenColCount) {
            container = table.rightFrozenHeaderDomContainer;
        }
    }
    return container;
}
function getTargetGroup(target) {
    while (target === null || target === void 0 ? void 0 : target.parent) {
        if (target.name === 'custom-container') {
            return target;
        }
        target = target.parent;
    }
    return { col: -1, row: -1, stage: null };
}

export { VTableVueAttributePlugin };
